@@ -379,9 +379,11 @@ class TestIntegerBase(unittest.TestCase):
         self.failUnless(isinstance(v1 >> v2, self.Integer))
         self.assertEqual(v1 >> v2, 0x08)
         self.assertEqual(v1 >> 1, 0x08)
-        self.assertEqual(v3 >> 1, -0x08)
         self.assertRaises(ValueError, lambda: v1 >> -1)
-        self.assertRaises(ValueError, lambda: v1 >> (2 ** 1000))
+        self.assertEqual(v1 >> (2 ** 1000), 0)
+        
+        self.assertEqual(v3 >> 1, -0x08)
+        self.assertEqual(v3 >> (2 ** 1000), -1)
 
     def test_in_place_right_shift(self):
         v1, v2, v3 = self.Integers(0x10, 1, -0x10)
@@ -438,10 +440,15 @@ class TestIntegerBase(unittest.TestCase):
         self.assertEqual(v1.get_bit(1), 1)
         self.assertEqual(v1.get_bit(v3), 1)
         self.assertEqual(v1.get_bit(8), 1)
-        self.assertEqual(v2.get_bit(0), 1)
-        self.assertEqual(v2.get_bit(8), 1)
+        self.assertEqual(v1.get_bit(9), 0)
+
+        self.assertRaises(ValueError, v1.get_bit, -1)
+        self.assertEqual(v1.get_bit(2 ** 1000), 0)
+
         self.assertRaises(ValueError, v2.get_bit, -1)
-        self.assertRaises(ValueError, v2.get_bit, 2 ** 1000)
+        self.assertRaises(ValueError, v2.get_bit, 0)
+        self.assertRaises(ValueError, v2.get_bit, 1)
+        self.assertRaises(ValueError, v2.get_bit, 2 * 1000)
 
     def test_odd_even(self):
         v1, v2, v3, v4, v5 = self.Integers(0, 4, 17, -4, -17)
@@ -673,6 +680,21 @@ def get_tests(config={}):
     except (ImportError, OSError), e:
         import sys
         sys.stdout.write("Skipping GMP tests (%s)\n" % str(e) )
+
+    try:
+        from Crypto.Math import _Numbers_custom as NumbersCustomModexp
+
+        class TestIntegerCustomModexp(TestIntegerBase):
+            def setUp(self):
+                self.Numbers = NumbersCustomModexp
+                self.Integer = NumbersCustomModexp.Integer
+                TestIntegerBase.setUp(self)
+
+        tests += list_test_cases(TestIntegerCustomModexp)
+    except (ImportError, OSError), e:
+        import sys
+        sys.stdout.write("Skipping custom modexp tests (%s)\n" % str(e) )
+
     tests += list_test_cases(TestIntegerGeneric)
     return tests
 
